@@ -11,11 +11,12 @@ import { contextSrv } from 'app/core/services/context_srv';
 import sortByKeys from 'app/core/utils/sort_by_keys';
 
 // Types
-import { PanelModel, GridPos } from './PanelModel';
+import { PanelModel, GridPos, panelAdded, panelRemoved } from './PanelModel';
 import { DashboardMigrator } from './DashboardMigrator';
-import { TimeRange, TimeZone } from '@grafana/data';
+import { TimeRange, TimeZone, AppEvent } from '@grafana/data';
 import { UrlQueryValue } from '@grafana/runtime';
-import { KIOSK_MODE_TV, DashboardMeta } from 'app/types';
+import { PanelEvents } from '@grafana/data';
+import { KIOSK_MODE_TV, DashboardMeta, CoreEvents } from 'app/types';
 import { toUtc, DateTimeInput, dateTime, isDateTime } from '@grafana/data';
 
 export interface CloneOptions {
@@ -215,15 +216,15 @@ export class DashboardModel {
 
     panel.setViewMode(fullscreen, this.meta.isEditing);
 
-    this.events.emit('view-mode-changed', panel);
+    this.events.emit(PanelEvents.viewModeChanged, panel);
   }
 
   timeRangeUpdated(timeRange: TimeRange) {
-    this.events.emit('time-range-updated', timeRange);
+    this.events.emit(CoreEvents.timeRangeUpdated, timeRange);
   }
 
   startRefresh() {
-    this.events.emit('refresh');
+    this.events.emit(PanelEvents.refresh);
 
     for (const panel of this.panels) {
       if (!this.otherPanelInFullscreen(panel)) {
@@ -233,7 +234,7 @@ export class DashboardModel {
   }
 
   render() {
-    this.events.emit('render');
+    this.events.emit(PanelEvents.render);
 
     for (const panel of this.panels) {
       panel.render();
@@ -306,7 +307,7 @@ export class DashboardModel {
 
     this.sortPanelsByGridPos();
 
-    this.events.emit('panel-added', panel);
+    this.events.emit(panelAdded, panel);
   }
 
   sortPanelsByGridPos() {
@@ -341,9 +342,9 @@ export class DashboardModel {
 
     // remove panels
     _.pull(this.panels, ...panelsToRemove);
-
+    panelsToRemove.map(p => p.destroy());
     this.sortPanelsByGridPos();
-    this.events.emit('repeats-processed');
+    this.events.emit(CoreEvents.repeatsProcessed);
   }
 
   processRepeats() {
@@ -363,7 +364,7 @@ export class DashboardModel {
     }
 
     this.sortPanelsByGridPos();
-    this.events.emit('repeats-processed');
+    this.events.emit(CoreEvents.repeatsProcessed);
   }
 
   cleanUpRowRepeats(rowPanels: PanelModel[]) {
@@ -596,7 +597,7 @@ export class DashboardModel {
   removePanel(panel: PanelModel) {
     const index = _.indexOf(this.panels, panel);
     this.panels.splice(index, 1);
-    this.events.emit('panel-removed', panel);
+    this.events.emit(panelRemoved, panel);
   }
 
   removeRow(row: PanelModel, removePanels: boolean) {
@@ -761,7 +762,7 @@ export class DashboardModel {
       this.sortPanelsByGridPos();
 
       // emit change event
-      this.events.emit('row-expanded');
+      this.events.emit(CoreEvents.rowExpanded);
       return;
     }
 
@@ -774,7 +775,7 @@ export class DashboardModel {
     row.collapsed = true;
 
     // emit change event
-    this.events.emit('row-collapsed');
+    this.events.emit(CoreEvents.rowCollapsed);
   }
 
   /**
@@ -798,12 +799,12 @@ export class DashboardModel {
     return rowPanels;
   }
 
-  on(eventName: string, callback: Function) {
-    this.events.on(eventName, callback);
+  on<T>(event: AppEvent<T>, callback: (payload?: T) => void) {
+    this.events.on(event, callback);
   }
 
-  off(eventName: string, callback?: Function) {
-    this.events.off(eventName, callback);
+  off<T>(event: AppEvent<T>, callback?: (payload?: T) => void) {
+    this.events.off(event, callback);
   }
 
   cycleGraphTooltip() {
@@ -911,7 +912,7 @@ export class DashboardModel {
 
   templateVariableValueUpdated() {
     this.processRepeats();
-    this.events.emit('template-variable-value-updated');
+    this.events.emit(CoreEvents.templateVariableValueUpdated);
   }
 
   expandParentRowFor(panelId: number) {
