@@ -1,12 +1,16 @@
 import angular from 'angular';
 import $ from 'jquery';
+// @ts-ignore
 import Drop from 'tether-drop';
+// @ts-ignore
 import baron from 'baron';
+import { PanelEvents } from '@grafana/data';
+import { getLocationSrv } from '@grafana/runtime';
 
 const module = angular.module('grafana.directives');
 
 const panelTemplate = `
-  <div class="panel-container">
+  <div class="panel-container" ng-class="{'panel-container--no-title': !ctrl.panel.title.length}">
       <div class="panel-header" ng-class="{'grid-drag-handle': !ctrl.panel.fullscreen}">
         <span class="panel-info-corner">
           <i class="fa"></i>
@@ -17,7 +21,7 @@ const panelTemplate = `
           <i class="fa fa-spinner fa-spin"></i>
         </span>
 
-        <panel-header class="panel-title-container" panel-ctrl="ctrl"></panel-header>
+        <panel-header class="panel-title-container" panel-ctrl="ctrl" aria-label="Panel Title"></panel-header>
       </div>
 
       <div class="panel-content">
@@ -38,14 +42,14 @@ module.directive('grafanaPanel', ($rootScope, $document, $timeout) => {
       const panelContent = elem.find('.panel-content');
       const cornerInfoElem = elem.find('.panel-info-corner');
       const ctrl = scope.ctrl;
-      let infoDrop;
-      let panelScrollbar;
+      let infoDrop: any;
+      let panelScrollbar: any;
 
       // the reason for handling these classes this way is for performance
       // limit the watchers on panels etc
       let transparentLastState = false;
       let lastHasAlertRule = false;
-      let lastAlertState;
+      let lastAlertState: boolean;
       let hasAlertRule;
 
       function mouseEnter() {
@@ -64,6 +68,12 @@ module.directive('grafanaPanel', ($rootScope, $document, $timeout) => {
         }
       }
 
+      function infoCornerClicked() {
+        if (ctrl.error) {
+          getLocationSrv().update({ partial: true, query: { inspect: ctrl.panel.id } });
+        }
+      }
+
       // set initial transparency
       if (ctrl.panel.transparent) {
         transparentLastState = true;
@@ -71,7 +81,7 @@ module.directive('grafanaPanel', ($rootScope, $document, $timeout) => {
       }
 
       // update scrollbar after mounting
-      ctrl.events.on('component-did-mount', () => {
+      ctrl.events.on(PanelEvents.componentDidMount, () => {
         if (ctrl.__proto__.constructor.scrollable) {
           const scrollRootClass = 'baron baron__root baron__clipper panel-content--scrollable';
           const scrollerClass = 'baron__scroller';
@@ -100,7 +110,7 @@ module.directive('grafanaPanel', ($rootScope, $document, $timeout) => {
         }
       });
 
-      ctrl.events.on('panel-size-changed', () => {
+      ctrl.events.on(PanelEvents.panelSizeChanged, () => {
         ctrl.calculatePanelHeight(panelContainer[0].offsetHeight);
         $timeout(() => {
           resizeScrollableContent();
@@ -108,7 +118,7 @@ module.directive('grafanaPanel', ($rootScope, $document, $timeout) => {
         });
       });
 
-      ctrl.events.on('view-mode-changed', () => {
+      ctrl.events.on(PanelEvents.viewModeChanged, () => {
         // first wait one pass for dashboard fullscreen view mode to take effect (classses being applied)
         setTimeout(() => {
           // then recalc style
@@ -121,7 +131,7 @@ module.directive('grafanaPanel', ($rootScope, $document, $timeout) => {
         }, 10);
       });
 
-      ctrl.events.on('render', () => {
+      ctrl.events.on(PanelEvents.render, () => {
         // set initial height
         if (!ctrl.height) {
           ctrl.calculatePanelHeight(panelContainer[0].offsetHeight);
@@ -197,6 +207,8 @@ module.directive('grafanaPanel', ($rootScope, $document, $timeout) => {
       elem.on('mouseenter', mouseEnter);
       elem.on('mouseleave', mouseLeave);
 
+      cornerInfoElem.on('click', infoCornerClicked);
+
       scope.$on('$destroy', () => {
         elem.off();
         cornerInfoElem.off();
