@@ -1,7 +1,6 @@
 import { ThunkAction, ThunkDispatch as GenericThunkDispatch } from 'redux-thunk';
 import { ActionOf } from 'app/core/redux';
 
-import { NavIndex } from './navModel';
 import { LocationState } from './location';
 import { AlertRulesState } from './alerting';
 import { TeamsState, TeamState } from './teams';
@@ -13,6 +12,11 @@ import { UsersState, UserState } from './user';
 import { OrganizationState } from './organization';
 import { AppNotificationsState } from './appNotifications';
 import { PluginsState } from './plugins';
+import { NavIndex } from '@grafana/data';
+import { ApplicationState } from './application';
+import { LdapState, LdapUserState } from './ldap';
+import { PanelEditorState } from '../features/dashboard/panel_editor/state/reducers';
+import { ApiKeysState } from './apiKeys';
 
 export interface StoreState {
   navIndex: NavIndex;
@@ -22,6 +26,7 @@ export interface StoreState {
   team: TeamState;
   folder: FolderState;
   dashboard: DashboardState;
+  panelEditor: PanelEditorState;
   dataSources: DataSourcesState;
   explore: ExploreState;
   users: UsersState;
@@ -29,6 +34,10 @@ export interface StoreState {
   appNotifications: AppNotificationsState;
   user: UserState;
   plugins: PluginsState;
+  application: ApplicationState;
+  ldap: LdapState;
+  ldapUser: LdapUserState;
+  apiKeys: ApiKeysState;
 }
 
 /*
